
(in-package :ystok.local-time)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  PRINT-TIME  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(setq dt @P6H15M)
(setq dt @P-6H15M)
(decode-duration dt)
(~dt *standard-output* dt)
(~dt *standard-output* dt nil nil 3 nil 7)
(dt-format-to-string dt :float nil :elements 3 :d 7)
(dt-format-to-string dt :float t :elements 3 :d 1)
(dt-format-to-string 1.04 :float t :elements 0 :d 2)
(dt-format-to-string 1.04 :float t :elements 3 :d 2)
(dt-format-to-string nil :float t :elements 3 :d 2)

(d-long-princ-to-string (lt:current-date) t nil)
(d-long-princ-to-string (lt:current-date) nil t nil)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  DURATION  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(decode-duration (encode-ym-duration -3 2))
(setq ym (encode-ym-duration -2))
(encode-ym-duration 3.2)

(setq ds (make-dt-duration :sec -600))
(setq ds (make-dt-duration :day -4 :sec 3600 :msec 100)) ;=> @P-3D22H59M59.9S
(setq ds (make-dt-duration :day -4 :sec 3600 :msec -100)) ;=> @P-3D23H0.1S 
(setq ds (make-dt-duration :sec 3600 :msec -100))

(setq ds (encode-dt-duration 10 3 2 30.5 100))

(format-duration *standard-output* ds)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  TIMEZONE  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;(defmacro daylight-savings-p (local-time) `(nth-value 1 (timezone ,local-time)))

(defmacro define-timezone (zone-name zone-file &key load)
  "Define ZONE-NAME (a symbol or a string) as a new timezone,
lazy-loaded from ZONE-FILE (a pathname designator relative to the
zoneinfo directory on this system). If LOAD is true, load immediately."
  (declare (ignore zone-name zone-file load))
  (error "not implemented"))

            
; Year 2003: Mar-30 3:00 -- Oct-26 2:00
(day-of-year 2003 3 1)
(%get-ymd-timezone-info 2003 3 30 2) ;=>10800, 0
(%get-ymd-timezone-info 2003 3 30 3) ;=>10800, 3600
(%get-ymd-timezone-info 2003 10 26 2);=>10800, 3600
(%get-ymd-timezone-info 2003 10 26 3);=>10800, 0

(%get-lt-timezone-info -60 720)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  CORE  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#+unused
(defun decode-gmtime (local-time)
  (let ((unix-time (unix-time local-time)))
    (multiple-value-bind (seconds minutes hours day month year wday yday dst)
        (unix-gmtime unix-time)
      (declare (ignore yday))
      (values (local-time-msec local-time)
              seconds minutes hours day (1+ month) (+ 1900 year) wday
              (= 1 dst) (unix-tz-displacement unix-time) "Not Implemented"))))

(defun local-time-hours (local-time)
  "return the local-time as a number of hours, rounded down"
  (let* ((day-hours (* 24 (local-time-day local-time))))
    (+ day-hours
       (floor (/ (local-time-sec local-time) 3600)))))

(defconstant +java-to-unix+ 1000)
(defun %java-to-local-time (java &optional (msec2 0))
  "Convert java timeval to local-time instance. Java is unix * 1000"
  (multiple-value-bind (sec msec) (floor java +java-to-unix+)
    (%unix-to-local-time sec (+ msec msec2))))
(defun %local-time-to-java (local-time)
  (multiple-value-bind (unix msec) (%local-time-to-unix local-time)
    (+ (* unix +java-to-unix+) msec)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  PARSE-TIME  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(parse-timestring "-14:25" :dt t)
(meta-parse-iso-8601 "2001-2- 14:9:3-10:")
(meta-parse-iso-8601 "2001-12- 14:9:16.0005-10:30" :junk-allowed t)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  OPERATIONS  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmethod local-time+22 ((x local-time) (y ym-duration))
 ;;; If x is the last day of the month or if the resulting month has fewer days
  ;; than the day component of x, then the result is the last day of the resulting
  ;; month. Otherwise, the result has the same day component as x.
  ;; NB: a la Oracle's ADD_MONTHS.
  (let ((months (duration-month y))
        (x-day (local-time-day x)))
    (multiple-value-bind (year month day) (%lt-day-ymd x-day)
      (declare (fixnum year month day))
      (let* ((days-in-begin-month (days-in-month year month))
             (days (cond ((> months 0)
                          (loop repeat months
                                sum (days-in-month year month)
                                if (= month 12) do (setq month 1  year (1+ year))
                                else do (incf month)))
                         ((< months 0)
                          (- (loop repeat (- months)
                                   if (= month 1) do (setq month 12  year (1- year))
                                   else do (decf month)
                                   sum (days-in-month year month))))
                         (0)))
             (days-in-end-month (days-in-month year month)))
        ;; year and month are resulting
        (make-local-time :day (+ x-day days
                                 (if (or (= day days-in-begin-month)	; last day
                                         (> day days-in-end-month))
                                     (- days-in-end-month day)
                                     0))
                         :sec (local-time-sec x)
                         :msec (local-time-msec x)
                         :zone (if (utc-time-p x) (local-time-zone x) nil))
) ) ) )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  UTILS  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(start-of-week (lt:current-date))
(finish-of-week (lt:current-date))
(start-of-10days (lt:current-date))
(finish-of-10days (lt:current-date))
(start-of-quarter @2005-01-31)
(finish-of-quarter (lt:current-date))
(finish-of-quarter @2005-01-01)
(integer-length +null-day+)
(integer-to-octets +null-day+ :length 3)
(integer-to-octets +null-day+ :length nil)

#+unused
(defun date-to-octets (date)
  (integer-to-octets (if date (local-time-day date) +null-day+)
                     :length 3))
#+unused
(defun octets-to-date (octets &optional (start 0) (end (+ start 3)))
  (let ((day (octets-to-signed octets start end)))
    (if (= day +null-date-day+) nil (make-date :day day))))
