;;; -*- Mode: Lisp -*-
;;; Local-Time - Examples and "informal" tests
;;; Copyright (c) 2002-2009 Dr. Dmitriy Ivanov. All rights reserved.

(in-package :local-time)
      
(defun test-lt-greg-isomorphic ()
  (declare (optimize (speed 3) (safety 1)))
  (dotimes (year 401)
    (declare (type fixnum year))
    (format t "~%;; Year: ~d" year)
    (dotimes (m 12)
      (declare (type fixnum m))
      (let ((month (1+ m)))
        (declare (type fixnum month))
        (dotimes (d (days-in-month year month))
          (let* ((day (1+ d))
                 (lt-day (%ymd-lt-day year month day)))
            (multiple-value-bind (year2 month2 day2)
                (%lt-day-ymd lt-day) ; (%gregorian-date local-day)
              (when (not (= year year2))
                (error "year wrong"))
              (when (not (= month month2))
                (error "month wrong"))
              (when (not (= day day2))
                (error "day wrong")))))))))

(defun test-encode-decode (list &optional (zones '(nil t 14400))
                                          (types '(local-time utc-time)))
 ;; encode-decode consistency
  (dolist (*default-encode-type* types)
   (dolist (*session-timezone* zones)
     (format t "~2%;;; *default-encode-type*=~S *session-timezone*=~S"
             *default-encode-type* *session-timezone*)
     (let (wrong-dl)
      (dolist (elt list)
       (setq wrong-dl nil)
       (let* ((lt (apply 'encode-local-time elt))
              (dl (multiple-value-list (decode-as-local lt t)))
              (du (multiple-value-list (decode-as-utc lt))))
         (format t "~% >> ~S => ~S~%    #<~A :day ~D :sec ~D :msec ~D~@[ :zone ~D~]>
 L> ~S~:[  ** wrong~]~% U> ~S~:[  ** wrong~]"
                 elt lt
                 (type-of lt) (local-time-day lt) (local-time-sec lt)
                 (local-time-msec lt)
                 (when (utc-time-p lt) (local-time-zone lt))
                 dl (equal (subseq elt 0 7) (subseq dl 0 7))
                 du (equal (subseq elt 0 7) (subseq du 0 7)))
         (unless (equal (subseq elt 0 7) (subseq dl 0 7))
           (setq wrong-dl t))
         ))
      (when wrong-dl
        (capi:beep-pane)
        (format t "~%<<< ** wrong decode-as-local **"))
    ))))

#||;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; EVALUATION ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(day-of-year 2001 3 1)
(%ymd-lt-day 2001 2 29 t)
(%lt-day-ymd most-negative-fixnum)
(test-lt-greg-isomorphic)
(test-encode-decode
 '((999 0 0 0 1 3 2000)
   (999 0 0 0 1 3 2000 18000)
   (999 0 0 0 1 3 2000 18000 utc-time) ; explicit a la from parser
 ))

(test-encode-decode
 '((999 0 0 2 30 3 2003)
   (999 0 0 3 30 3 2003))
 ))

(lt::get-win32-system-time)
(lt::get-win32-local-time)
(setq ut (get-universal-time))
(setq ut (encode-universal-time 0 0 0 1 3 2000))
(setq ut (encode-universal-time 0 25 4 4 9 2003))
(setq ut (universal-time tsl))
(decode-universal-time ut)

(setq xt (unix-time tsl))
(setq xt (unix-mktime 103 88 4 3 0 0))
(unix-gmtime xt)
(unix-localtime xt)
(unix-timezone xt)
(unix-timezone)
(win32-timezone)
(get-win32-timezone-information)
(unix-time lt)
(timezone lt)
libc-timezone ;=> -14400
libc-daylight ;=> 0

(setq cts1 (local-time :universal (+ (get-universal-time) (unix-timezone)) :zone nil))
(setq cts2 (current-timestamp 3 'local-time))
(setq cts3 (current-timestamp 3 'utc-time))

(setq d (current-date))
(setq dl (current-date 'local-time))
(setq du (current-date 'utc-time))

(setq lt (localtimestamp))
(setq tsl (current-timestamp nil 'local-time))
(setq tsu (current-timestamp nil 'utc-time))
(decode-as-local tsl)
(decode-as-local tsu nil)
(ts-format-to-string tsu :timezone 3600 :universal t)
(decode-as-utc tsu)
(local-time-adjust tsu 3600 tsl)
(timezone tsu)
(time-component tsu :timezone-hour) 'utc-time)

(setq lt (local-time :universal (get-universal-time) :zone 14400))
(setq lt (local-time :unix xt))

;;; DST changing 2003-03-30 03:00
(setq d (encode-date 2003 3 30))
(setq lt  (encode-local-time 0 0 30 2 30 3 2003))
(setq lt3 (encode-local-time 0 0 30 3 30 3 2003))
(setq ut  (encode-local-time 0 0 30 2 30 3 2003 t 'utc-time))
(setq ut3 (encode-local-time 0 0 30 3 30 3 2003 t 'utc-time))
(local-time< d lt)
(local-time< d ut)
(local-time= lt lt3) ;=> nil
(local-time= ut ut3) ;=> t

(local-time= (encode-date 2003 3 30) (encode-local-time 0 0 0 0 30 3 2003)) ;=> T
(local-time= (encode-date 2003 3 29)
             (encode-local-time 0 0 0 0 29 3 2003 t 'utc-time)) ;=> T
(local-time= (encode-date 2003 3 30)
             (encode-local-time 0 0 0 0 30 3 2003 t 'utc-time)) ;=> T
(local-time= (encode-date 2003 3 31)
             (encode-local-time 0 0 0 0 31 3 2003 t 'utc-time)) ;=> T

(timezone d)  ;=> 10800, -3600  DI 2011-Sep-10: Before was 14400, 3600?!
(timezone lt) ;=> 10800, -3600
(timezone ut) ;=> 10800, nil
(timezone (encode-date 1979 3 30)) ;=> 14400, NIL
(timezone (encode-date 2011 3 27)) ;=> 10800, -3600
(timezone (encode-date 2011 3 28)) ;=> 14400, 0
(timezone (encode-date 2012 1 1)) ;=> 14400, 0

(floor -14400 +secs-per-day+) ;=> -1, 72000

(local-time- d lt)	;=> @P-2H30M
(local-time- d lt3)	;=> @P-3H30M
(local-time- d ut)	;=> @P-2H30M
(local-time- d ut3)	;=> @P-2H30M

(local-time- lt3 ut3)	;=> @P0D !!! Anomaly
(local-time- lt3 lt)	;=> @P1H
(local-time- ut3 lt)	;=> @P0D

(local-time- lt  ut)	;=> @P0H
(local-time- lt3 ut)	;=> @P0H
(local-time- ut3 ut)	;=> @P0H


;;; DST changing 2003-10-26 02:00 
(setq d (encode-date 26 10 2003))
(setq lt2 (encode-local-time 0 0 30 2 26 10 2003))
(setq lt3 (encode-local-time 0 0 30 3 26 10 2003))
(setq ut  (encode-local-time 0 0  0 0 26 10 2003 t 'utc-time))
(setq ut1 (encode-local-time 0 0  0 1 26 10 2003 t 'utc-time)) ; =d
(setq ut2 (encode-local-time 0 0 30 2 26 10 2003 t 'utc-time))
(setq ut3 (encode-local-time 0 0 30 3 26 10 2003 t 'utc-time))
(local-time< d lt2)   ;=> t
(local-time< d ut)   ;=> nil !!! ; We've got a comparison anomaly  
(local-time= d ut)   ;=> t   (=> nil when (timezone date) is computed
(local-time> d ut)   ;=> t           per 12:00 instead 00:00!)
(local-time= d ut1)  ;=> nil   
(local-time= lt2 ut2)  ;=> t
(local-time= lt3 ut3)  ;=> t

(local-time= (encode-date 25 10 2003) (encode-local-time 0 0 0 0 25 10 2003)) ;=> T
(local-time= (encode-date 26 10 2003) (encode-local-time 0 0 0 0 26 10 2003)) ;=> T
(local-time= (encode-date 27 10 2003) (encode-local-time 0 0 0 0 27 10 2003)) ;=> T
(local-time= (encode-date 25 10 2003)
             (encode-local-time 0 0 0 0 25 10 2003 t 'utc-time)) ;=> T
(local-time= (encode-date 26 10 2003)
             (encode-local-time 0 0 0 0 26 10 2003 t 'utc-time)) ;=> T
(local-time= (encode-date 27 10 2003)
             (encode-local-time 0 0 0 0 27 10 2003 t 'utc-time)) ;=> T

(timezone d)  ;=> 10800, 0
(timezone lt2) ;=> 14400, 3600
(timezone ut) ;=> 14400, nil
(floor -10800 +secs-per-day+) ;=> -1, 75600

(local-time- d lt2)	;=> @P-2H30M
(local-time- d ut)	;=> @P0H
(local-time- d ut1)	;=> @P-1H
(local-time- ut1 ut)	;=> @P1H
(local-time- ut2 ut1)	;=> @P1H30M
(local-time- ut3 ut2)	;=> @P2H  !!!
(local-time- lt3 lt2)	;=> @P1H

(local-time- lt3 ut3)	;=> @P0D  !!! Anomaly
(local-time- lt3 lt2)	;=> @P1H
(local-time- ut3 lt2)	;=> @P2H

(local-time- d lt3)	;=> @P-3H30M
(local-time- d ut3)	;=> @P-4H30M

(multiple-value-list (decode-as-local lt)) ; => (0 0 0 2 1 1 2000 5 NIL 10800)
(multiple-value-list (decode-as-utc lt))
(week-day-number (encode-local-time 0 0 0 1 30 3 2003 t 'local-time))
(week-day-number (encode-local-time 0 0 0 1 30 3 2003 t 'utc-time))
(week-day-name lt)
(week lt)
(local-time-hms (encode-local-time 0 0 0 1 30 3 2003 t 'local-time))
(local-time-hms (encode-local-time 0 0 0 1 30 3 2003 t 'utc-time))
(local-time-hms (current-date))

(setq lt (encode-local-time 0 0 -30 -30 30 3 2003))
(setq ut (encode-local-time 0 0 30 3 30 3 2003 (* 5 3600) 'utc-time))
(setq lt2 (encode-local-time 0 0 30 2 30 3 2003))
(setq ut2 (encode-local-time 0 0 30 2 30 3 2003 t 'utc-time))
(multiple-value-bind (day sec) (local-time-adjust ut (* -2 3600))
  (make-local-time :day day :sec sec :zone (* -2 3600)))
(local-time-adjust lt (* -2 3600) lt2)
(local-time-adjust lt (* -2 3600) ut2)
(local-time-adjust ut (* -2 3600) ut2)
(local-time-adjust ut (* -2 3600) lt2)
(local-time-adjust d (* -2 3600))

(setq lt (parse-timestring "2000-01-01 02:00:00.32"))
(setq lt (parse-timestring "2000-01-01 23:00-3"))
(setq lt (parse-timestring "2000-03-01 00:00Z"))
(setq lt (parse-timestring "2000-03-01 00:00:00-00")) ;; 0
(format-timestring nil tsl t t)
(format-timestring *standard-output* lt t t 4 4 0 0 #\T)
(%lt-day-dow 5)
(format-timestring nil lt t t :ywd)

(meta-parse-iso-8601 "2000-01-10T" :time-elements nil)
(meta-parse-iso-8601 "14:22:20.+6" :date-elements nil :timezone-p t)
(decode-as-local lt)
(decode-as-utc lt)
(format nil "~,,'.@/local-time:@/" (parse-iso-8601-time "2000-05-10"))
(format nil "~2,,,,' @/local-time:@/" (my-get-local-time))
(format nil "~0,@/local-time:~@/" (get-universal-time))

'@2000-01-01T02:00:00-03
'@"2000-01-01 02:00:00-03"

(local-time= @2000-01-01T02:00:00.2 @2000-01-01T02:00:00.2+3 @2000-01-01T02:00:00.3+3)
(%local-time= @2000-01-01T02:00:00.3Z @2000-01-01T05:00:00.3+3)
(local-time/= @2000-01-01T02:00:00 @2000-01-01T02:00:00.1+3 @2000-01-01T02:00:00.2+3)

(local-time<2 @2000-01-01T02:00:00 @2000-01-01T02:00:01)
(local-time< @2000-01-01T02:00:00 @2000-01-01T02:00:01 @2000-01-01T02:00:02+3)
(local-time< @2000-01-01T02:00:01 @2000-01-01T02:00:00 @2000-01-01T02:00:02+3)

(local-time<= @2000-01-01T02:00:00 @2000-01-01T02:00:01 @2000-01-01T02:00:01+3)
(local-time<= @2000-01-01T02 @2000-01-01T02:00:00 @2000-01-01T02:00:02+3)

(meta-parse-local-time "97-01-01 02:00:00,32")
(meta-parse-local-time "1.2.7 23:00:00-3" :date-format :dmy :date-separator #\.
                       :century-digits 2)
;; AM/PM
(meta-parse-local-time "11:10:00 PM" :date-format nil :time-format 12)
(meta-parse-local-time "1.2.1997 11:10:00-3 PM" :date-format :dmy :date-separator #\.
                                                :time-format 12)

(~d *standard-output* tsl nil nil :dmy 2 #\.)
(~t *standard-output* lt t nil nil 3)
(~t *standard-output* tsu nil t 12 4)
(~t *standard-output* (universal-time tsu) t nil 12 4)
(~ts *standard-output* lt nil nil :dmy 2)
(~ts *standard-output* tsu nil nil :dmy nil nil 12 4)
(~ts *standard-output* tsu nil nil :dmy nil nil 12 0)
(~ts *standard-output* (universal-time tsu) t nil 12 4)
(~ts-format-to-string tsu :date-format :dmy :time-elements 4)


(local-time-to-string
 (month-first-day (parse-timestring "2000-05-10 00:00:00-06")))


(multiple-value-list (local-time-ymd (parse-timestring "2000-03-25 00:00:00+06")))
(time-component (parse-timestring "1971-03-25 00:00:00-06") :year)
(time-component (parse-timestring "2004-02-29 00:00:00-06") :year)
(time-component (parse-timestring "2004-03-01 00:00:00-06") :year)
(time-component (parse-timestring "2004-02-28 00:00:00-06") :year)
(time-component (parse-timestring "2003-05-31 00:00:00-06") :year)

(time-component (parse-timestring "2000-12-31 00:00:00-06") :year)
(time-component (parse-timestring "2000-03-01 00:00:00-06") :year)
(time-component (parse-timestring "1999-12-31 00:00:00-06") :year)
(time-component (parse-timestring "1999-02-29 00:00:00-06") :year)

(time-component lt :week-day-keyword)
(setq ym (parse-duration "8Y6M"))
(setq ym (parse-duration "-8y6M"))
(parse-duration "6M")
(decode-duration ym)

(parse-duration "5W4DT3H2M1,1S")
(setq dt (parse-duration "4DT3H2M1,1S"))
(parse-duration "4D3H2M1,1S")
(parse-duration "3H2M1,1S")
(parse-duration "T2M1,1S")
(parse-duration "2M1,1S")
(parse-duration "1,1S")
(parse-duration "6.6y5.5M")
(setq ds (parse-duration "-4DT3H2M1,1S"))
(parse-duration "4DT3.5H2M1,1S")
(parse-duration "4DT3.5H2M1S" :junk-allowed nil)
(parse-duration "4DT3.5He" :junk-allowed t)
(decode-duration dt)
(setq ds (encode-dt-duration -3.555555))
(decode-duration -3.555555)
(setq ds (encode-dt-duration -3.51))
(decode-duration -3.51)
(format-duration nil .01) ;=> "P-14M24S"
(+ (/ 24.0 +secs-per-day+) (/ 14.0 60 24)) ;=>2093760

(%get-day-sec-msec -4.499995)
(%number-day-sec-msec -4.499995)
(setq dt @P4D12H)
(local-time<2 @P4D12H 4.499995)
(local-time= @P4D12H @P4,5D @P108H 4.5)
(local-time/= @P4D12H 4.499999 @P4,51D)

(setq lt (encode-local-time 0 0 30 1 28 2 2003))
(setq ut (encode-local-time 0 0 30 1 28 2 2003 t 'utc-time))
(local-time+2 lt @P-1M)
(local-time+2 ut @P2M)
(local-time+22 lt @P2M)
(local-time+22 ut @P2M)

(setq d (encode-date 28 2 2003))
(local-time+2 @P3Y2M d)

(time (let ((l (copy-local-time ut)))
        (dotimes (i 1000 l) (setq l (local-time+2 l @P1Y)))))
(time (let ((l (copy-local-time ut)))
        (dotimes (i 1000 l) (setq l (local-time+22 l @P1Y)))))

(setq lt (local-time+ tsl 1.5))
(setq lt (local-time+ tsu 1.5))

(local-time+ @P4D12H 1.5)
(local-time+ d @P-1D1H)
(local-time+ d 2.5)
(local-time- d @P1D1H)
(local-time- d 2.5)
(setq d1 (let ((*default-encode-type* 'utc-time)) (local-time+ d @P-1D1H)))

(duration* @P4D3H2M1S 0.599999)

(duration/ @P8Y6M 7)
(duration/ @P8Y6M @P1Y2M)
(duration/ @P4D3H2M1S 2)
(duration/ @P4D3H @P2D)  ;=> 33/16
(duration/ 10 @P2D)

(dolist (month (month-span (parse-timestring "2000-02-29 00:00:00-06")
                           (parse-timestring "P1D")))
  (format t (local-time-to-string month))
  (format t "~%"))

(let ((stream (make-string-output-stream)))
  (format-timestring stream (parse-timestring "2000-02-29 00:00:00-06"))
  (get-output-stream-string stream))

(local-time-to-string (parse-timestring "2000-02-20 01:30:00-06") :pretty t)
(local-time-to-string (parse-timestring "2000-02-20 01:30:00-06") :short-pretty t)

(local-time-to-string
 (%universal-to-local-time
  (encode-universal-time 0 0 0 3 4 2000)))

(let ((stream (make-string-output-stream)))
  (format-timestring stream (parse-timestring "2000-05-02 00:00:00-06"))
  (get-output-stream-string stream))

(let ((stream (make-string-output-stream)))
  (format-timestring stream (parse-timestring "2000-03-31 00:00:00-06"))
  (get-output-stream-string stream))

(let ((stream (make-string-output-stream)))
  (format-timestring stream (parse-timestring "1999-04-01 11:32:21-06"))
  (get-output-stream-string stream))

(local-time= (parse-timestring "1999-04-01 11:32:21-06")
              (parse-timestring "1999-04-01 11:32:21-06"))

(multiple-value-list (local-time-hms (local-time::%make-local-time :day 0 :sec 7724 :msec 0)))
(read-n-characters 88 (make-string-input-stream "P100EDEE")
                   :keep-from 2 :keep-to 5)
(parse-timestring "P10D23H60M60S")
(days-in-month 1 2)
(truncate (+ (* 2003 12) 2  (1- 0)) 12)

;;; Week

(%lt-day-week (+ (local-time-day (current-date)) 1))
(%lt-day-week (local-time-day @2005-5-30))
(time-component (current-date) :week)
(time-component @2005-1-3 :week)
(format-timestring nil (current-date) t nil :ywd)
(~ts *standard-output* (current-date) t nil :yw)
(ts-format-to-string (current-date) :date-format :yw :date-separator 0)
(week @1994-12-31 t)	;=> 53	;IS0 => 52,1994
(week @1995-1-1 t)	;=>  1	;ISO => 52,1994
(week @1996-12-31 )	;=> 53	;ISO =>  1,1997
 (week @1997-01-01 t)

(week @1984-01-01 )	;=>  1	; ISO=> 52,1983
(week @1984-12-31 nil)	;=> 54
(week @1984-12-31 t)	;=> 1,1985
(ystok.local-time.test::test-non-iso-week 2001)

(make-date :day (%ywd-lt-day 1994 52 6)) ;=> @1994-12-31
(make-date :day (%ywd-lt-day 1994 52 7)) ;=> @1995-01-01
(make-date :day (%ywd-lt-day 1995 1 1))  ;=> @1995-01-02

(week @1998-12-31 t)	; ISO=> 53,1998
(make-date :day (%ywd-lt-day 1998 53 7))  ;=> @1995-01-02

(setq d (encode-date 1998 -53 24))
(setq d @1998-W11-4)

||#
