;;; -*- Mode: Lisp -*-
;;; Ystok-Local-Time - Test suite
;;; Copyright (c) 2002-2013 Dr. Dmitriy Ivanov. All rights reserved.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Tests are based on
;;; 1. CLSQL, test-time.lisp 2004-05-23 10:51:02Z kevin
;;; 2. UncommonSQL, Copyright (c) 2000, 2001 onShore Development, Inc.
;;; 3. ACL-COMPAT tester (slightly improved public version)

;(pushnew :tester *features*)		; needed to load acl-tester from ACL-COMPAT
;(load #P"PROJECTS:ynet;acl-compat;defsys")
;(lw:compile-system 'ACL-COMPAT :load t)
;(load #P"PROJECTS:ynet;acl-compat;acl-compat.asd")
;(asd:oos :load 'acl-compat)

(defpackage :ystok.local-time.test
 (:use :common-lisp :ystok.local-time)	;:test
 (:export #:do-tests))

(in-package :ystok.local-time.test)

(defun test-non-iso-week (low &optional (up (+ low 20)))
 ;;; Ensure that a later data cannot have a smaller week number (for non-ISO week numbers)
  (loop with result = t
        for year upfrom low upto up
        do
        (format t "~%;; Year: ~d" year)
        (dotimes (m 12)
          (declare (type fixnum m))
          (let ((month (1+ m)))
            (declare (type fixnum month))
            (dotimes (d (days-in-month year month))
              (let* ((d1 (+ d 1))
                     (d2 (+ d 2))
                     (date1 (encode-date year month d1 nil))
                     (date2 (encode-date year month d2 nil))
                     (w1 (week date1 nil))
                     (w2 (week date2 nil)))
                (when (and (= (time-component date1 :year) (time-component date2 :year))
                           (> w1 w2))
                  (format t
                     "~%;** test-weeks fails ~a vs ~a: week1 ~d > week2 ~d"
                     date1 date2 w1 w2)
                  (setq result nil))))))
        finally (return result)))

(defun do-tests ()
 (let ((test:*break-on-test-failures* nil))
  (test:with-tests (:name :local-time)

    ;; Test the boundaries of Local Time with granularity of 1 year
    (test:test 50
      (let ((sec-in-year (* 60 60 24 365))
            (year (time-component (make-local-time) :year)))
        (dotimes (n 50 n)
          (let ((date (make-local-time :sec (* n sec-in-year))))
          (unless (= (+ year n) (time-component date :year))
            (return n)))))
      :reported-form :time-component/7)

    ;; Test format-timestring
    (test:test 2000
      (flet ((%year (str) (parse-integer (subseq str 0 4))))
        (let ((second-in-year (* 60 60 24 365)))
          (dotimes (n 2000 n)
            (let* ((second (* -1 n second-in-year))
                   (date (encode-local-time 0 second 0 0 1 1 2525)))
              (unless (= (%year (format-timestring nil date))
                         (time-component date :year))
                (return n))))))
      :reported-form :format-timestring/9)

    (test:test-true (test-non-iso-week 2001)
      :reported-form :test-non-iso-weeks)

    ;; Conversion between MJD and Gregorian
    #|(test:test :time/10
        (dotimes (base 10000 base)
          (unless (= (apply #'lt:gregorian-to-mjd (lt:mjd-to-gregorian base))
                     base)
            (return base)))
        10000)|#
  
    ;; Add 90 seconds
    (test:test-true
      (let* ((now (current-timestamp))
             (add-time (lt::copy-local-time now)))
        (incf (local-time-sec add-time) 90)
        (local-time= (local-time+ now (encode-dt-duration 0 0 0 90))
                     add-time))
      :reported-form :time/11)

    #|;; Roll by minutes: +900
     (test:test :time/12
       (let ((now (lt:get-time)))
         (lt:local-time= (lt:time+ now (lt:make-duration :minute 900))
                         (lt:roll now :minute 900)))
       t)
  
     ;; Roll by minutes: +900
     (test:test :time/13
       (let* ((now (lt:get-time))
              (add-time (lt:time+ now (lt:make-duration :minute 9000)))
              (roll-time (lt:roll now :minute 9000)))
         (lt:local-time= add-time roll-time))
       t)|#

    ;; time-to-octets, octets-to-time
    (let ((date (current-date)))
      (test:test date
        (octets-to-time (time-to-octets date))
        :test 'date=)
      (dolist (length '(nil 3 4))
        (test:test date
          (octets-to-time (time-to-octets date length) 0 length)
          :test 'date=)))

    (let ((now (localtimestamp)))
      (test:test now
        (octets-to-time (time-to-octets now))
        :test 'local-time=)
      (dolist (length '(nil 8 9))
        (test:test now
          (octets-to-time (time-to-octets now length) 0 length)
          :test 'local-time=)))

    (test:test nil
      (octets-to-time (time-to-octets nil))
      :reported-form :time-to-octets/null/date)		; #(0 0 128)
    (test:test nil
      (octets-to-time (time-to-octets nil 8) 0 8)	; #(0 0 128 255 255 255 255 255)
      :reported-form :time-to-octets/null/time)

    (and (= 0 test:*test-errors* test:*test-unexpected-failures*)
         (/= 0 test:*test-successes*)))))


#|;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(ystok.local-time.test:do-tests)

#+sbcl (compile-file #P"PROJECTS:ylib;local-time;test.lisp")
#+sbcl (load #P"PROJECTS:ylib;local-time;test.fasl")

;; relations of intervals
(test:test-true
 (let* ((time-1 (lt:parse-timestring "2002-01-01 10:00:00"))
        (time-2 (lt:parse-timestring "2002-01-01 11:00:00"))
        (time-3 (lt:parse-timestring "2002-01-01 12:00:00"))
        (time-4 (lt:parse-timestring "2002-01-01 13:00:00"))
        (interval-1 (lt:make-interval :start time-1 :end time-2))
        (interval-2 (lt:make-interval :start time-2 :end time-3))
        (interval-3 (lt:make-interval :start time-3 :end time-4))
        (interval-4 (lt:make-interval :start time-1 :end time-3))
        (interval-5 (lt:make-interval :start time-2 :end time-4))
        (interval-6 (lt:make-interval :start time-1 :end time-4)))
   (flet ((my-assert (number relation i1 i2)
            (declare (ignore number))
            (let ((found-relation (lt:interval-relation i1 i2)))
              (equal relation found-relation))))
     (and 
      (my-assert 1 :contains interval-1 interval-1)
      (my-assert 2 :precedes interval-1 interval-2)
      (my-assert 3 :precedes interval-1 interval-3)
      (my-assert 4 :contained interval-1 interval-4)
      (my-assert 5 :precedes interval-1 interval-5)
      (my-assert 6 :contained interval-1 interval-6)
      (my-assert 7 :follows interval-2 interval-1)
      (my-assert 8 :contains interval-2 interval-2)
      (my-assert 9 :precedes interval-2 interval-3)
      (my-assert 10 :contained interval-2 interval-4)
      (my-assert 11 :contained interval-2 interval-5)
      (my-assert 12 :contained interval-2 interval-6)
      (my-assert 13 :follows interval-3 interval-1)
      (my-assert 14 :follows interval-3 interval-2)
      (my-assert 15 :contains interval-3 interval-3)
      (my-assert 16 :follows interval-3 interval-4)
      (my-assert 17 :contained interval-3 interval-5)
      (my-assert 18 :contained interval-3 interval-6)
      (my-assert 19 :contains interval-4 interval-1)
      (my-assert 20 :contains interval-4 interval-2)
      (my-assert 21 :precedes interval-4 interval-3)
      (my-assert 22 :contains interval-4 interval-4)
      (my-assert 23 :overlaps interval-4 interval-5)
      (my-assert 24 :contained interval-4 interval-6)
      (my-assert 25 :follows interval-5 interval-1)
      (my-assert 26 :contains interval-5 interval-2)
      (my-assert 27 :contains interval-5 interval-3)
      (my-assert 28 :overlaps interval-5 interval-4)
      (my-assert 29 :contains interval-5 interval-5)
      (my-assert 30 :contained interval-5 interval-6)
      (my-assert 31 :contains interval-6 interval-1)
      (my-assert 32 :contains interval-6 interval-2)
      (my-assert 33 :contains interval-6 interval-3)
      (my-assert 34 :contains interval-6 interval-4)
      (my-assert 35 :contains interval-6 interval-5)
      (my-assert 36 :contains interval-6 interval-6))))
 :reported-form :time/intervals)

;; adjacent intervals in list
(test:test :time/2
  (let* ((interval-list nil)
         (time-1 (lt:parse-timestring "2002-01-01 10:00:00"))
         (time-3 (lt:parse-timestring "2002-01-01 12:00:00"))
         (time-4 (lt:parse-timestring "2002-01-01 13:00:00")))
    (setf interval-list 
	  (lt:interval-push interval-list (lt:make-interval :start time-1 :end time-3
						      :type :open)))
    (setf interval-list 
	  (lt:interval-push interval-list (lt:make-interval :start time-3 :end time-4
						      :type :open)))
    (lt:interval-relation (car interval-list) (cadr interval-list)))
  :precedes)

;; nested intervals in list
(test:test :time/3
    (let* ((interval-list nil)
	   (time-1 (lt:parse-timestring "2002-01-01 10:00:00"))
	   (time-2 (lt:parse-timestring "2002-01-01 11:00:00"))
	   (time-3 (lt:parse-timestring "2002-01-01 12:00:00"))
	   (time-4 (lt:parse-timestring "2002-01-01 13:00:00")))
      (setf interval-list 
	    (lt:interval-push interval-list (lt:make-interval :start time-1 
							:end time-4 
							:type :open)))
      (setf interval-list 
	    (lt:interval-push interval-list (lt:make-interval :start time-2 
							:end time-3 
							:type :closed)))
      (let* ((interval (car interval-list))
	     (interval-contained 
	      (when interval (car (lt:interval-contained interval)))))
	(when (and interval interval-contained)
	  (and (lt:local-time= (lt:interval-start interval) time-1)
	       (lt:local-time= (lt:interval-end interval) time-4)
	       (eq (lt:interval-type interval) :open)
	       (lt:local-time= (lt:interval-start interval-contained) time-2)
	       (lt:local-time= (lt:interval-end interval-contained) time-3)
	       (eq (lt:interval-type interval-contained) :closed)))))
  t)

;; interval-edit - nonoverlapping
(test:test :time/4
    (let* ((interval-list nil)
	   (time-1 (lt:parse-timestring "2002-01-01 10:00:00"))
	   (time-2 (lt:parse-timestring "2002-01-01 11:00:00"))
	   (time-3 (lt:parse-timestring "2002-01-01 12:00:00"))
	   (time-4 (lt:parse-timestring "2002-01-01 13:00:00")))
      (setf interval-list (lt:interval-push interval-list (lt:make-interval :start time-1 :end time-2 :type :open)))
      (setf interval-list (lt:interval-push interval-list (lt:make-interval :start time-3 :end time-4 :type :closed)))
      (setf interval-list (lt:interval-edit interval-list time-1 time-1 time-3))
      ;; should be time-3 not time-2 
      (lt:local-time= (lt:interval-end (car interval-list)) time-3))
  t)

;; interval-edit - overlapping
(test:test :time/5
    (let* ((interval-list nil)
	   (time-1 (lt:parse-timestring "2002-01-01 10:00:00"))
	   (time-2 (lt:parse-timestring "2002-01-01 11:00:00"))
	   (time-3 (lt:parse-timestring "2002-01-01 12:00:00"))
	   (time-4 (lt:parse-timestring "2002-01-01 13:00:00")))
      (setf interval-list (lt:interval-push interval-list (lt:make-interval :start time-1 :end time-2 :type :open)))
      (setf interval-list (lt:interval-push interval-list (lt:make-interval :start time-2 :end time-4 :type :closed)))
      (let ((pass t))
	(handler-case 
	    (progn 
	      (setf interval-list 
		    (lt:interval-edit interval-list time-1 time-1 time-3))
	      (setf pass nil))
	  (error nil))
	pass))
  t)

;; interval-edit - nested intervals in list
(test:test :time/6
    (let* ((interval-list nil)
	   (time-1 (lt:parse-timestring "2002-01-01 10:00:00"))
	   (time-2 (lt:parse-timestring "2002-01-01 11:00:00"))
	   (time-3 (lt:parse-timestring "2002-01-01 12:00:00"))
	   (time-4 (lt:parse-timestring "2002-01-01 13:00:00"))
	   (time-5 (lt:parse-timestring "2002-01-01 14:00:00"))
	   (time-6 (lt:parse-timestring "2002-01-01 15:00:00")))
      (setf interval-list (lt:interval-push interval-list (lt:make-interval :start time-1 :end time-6 :type :open)))
      (setf interval-list (lt:interval-push interval-list (lt:make-interval :start time-2 :end time-3 :type :closed)))
      (setf interval-list (lt:interval-push interval-list (lt:make-interval :start time-4 :end time-5 :type :closed)))
      (setf interval-list (lt:interval-edit interval-list time-1 time-1 time-4))
      ;; should be time-4 not time-6 
      (lt:local-time= (lt:interval-end (car interval-list)) time-4))
  t)
||#
